@@ -141,8 +141,8 @@ for ipv6 in yes no ; do
         iptables-save | grep '^-' > $TESTTMP/ipt.enable
         ip6tables-save | grep '^-' > $TESTTMP/ip6t.enable
 
-        do_extcmd "0" null $RELTESTPATH/lib/ufw/ufw-init stop
-        do_extcmd "0" null $RELTESTPATH/lib/ufw/ufw-init start
+        do_extcmd "0" null $TESTPATH/lib/ufw/ufw-init stop
+        do_extcmd "0" null $TESTPATH/lib/ufw/ufw-init start
         iptables-save | grep '^-' > $TESTTMP/ipt.start
         ip6tables-save | grep '^-' > $TESTTMP/ip6t.start
 
